//=============================================================================
// Belt.
//=============================================================================
class Belt extends TournamentPickup;

#exec AUDIO IMPORT FILE="Sounds\HitWallSound1.wav" 
#exec AUDIO IMPORT FILE="Sounds\HitWallSound2.wav" 
#exec AUDIO IMPORT FILE="Sounds\HitWallSound3.wav" 

var UT_ShieldBeltEffect MyEffect;
var() string TeamFireTextureStrings[4];
var() string TeamTextureStrings[4];
var firetexture TeamFireTextures[4];
var texture TeamTextures[4];
var int TeamNum;
var int NumPoints;
var int NumShields;
var int ShieldNum;

var Sound HitSound1;
var Sound HitSound2;
var Sound HitSound3;

var Belt NextBelt;

function PostBeginPlay()
{
	local NavigationPoint NP;
        
	Super.PostBeginPlay();
	for (NP = Level.NavigationPointList; NP != None; NP = NP.NextNavigationPoint)
	{
		if (NP.IsA('PathNode'))
			NumPoints++;
	}
}

State Sleeping
{
Begin:
	SetPhysics( PHYS_Rotating );
	Sleep( RespawnTime );
	MoveBelt();
	Sleep( Level.Game.PlaySpawnEffect(self) );
	NextBelt=Spawn(Class,,,Location);
	NextBelt.ShieldNum = ShieldNum;
	NextBelt.NumPoints = NumPoints;
	NextBelt.NumShields = NumShields;
	Destroy();
}

function MoveBelt()
{
	local NavigationPoint NP;
	local int i, RandomPoint, Ratio;
 
	Ratio = NumPoints/NumShields + 1;
	RandomPoint = Rand(ratio)*NumShields + ShieldNum;
	if (RandomPoint >= NumPoints)
		RandomPoint = ShieldNum;

	for (NP = Level.NavigationPointList; NP != None; NP = NP.NextNavigationPoint)
	if ( NP.IsA('PathNode') )
	{
		If (i == RandomPoint)
		{
			SetLocation(NP.Location);
			Return;
		}
		i++;
	}
}

event float BotDesireability( pawn Bot )
{
	return MaxDesireability;
}

function bool HandlePickupQuery( inventory Item )
{
	local Inventory I;

	if (item.class == class) 
	{
		// remove other armors
		for ( I=Owner.Inventory; I!=None; I=I.Inventory )
			if ( I.bIsAnArmor && (I != self) )
				I.Destroy();
	}

	return Super.HandlePickupQuery(Item);
}

function ArmorImpactEffect(vector HitLocation)
{ 
	if ( MyEffect != None )
	{
		MyEffect.ScaleGlow = 4.0;
		MyEffect.Fatness = 255;
		SetTimer(0.8, false);
	}
}

function Timer()
{
	if ( MyEffect != None )
	{
		MyEffect.Fatness = MyEffect.Default.Fatness;
		SetEffectTexture();
	}
}

function int ArmorAbsorbDamage(int Damage, name DamageType, vector HitLocation)
{
	if ( DamageType != 'Drowned' )
		ArmorImpactEffect(HitLocation);
	if( (DamageType!='None') && ((ProtectionType1==DamageType) || (ProtectionType2==DamageType)) )
		return 0;
	if (DamageType=='Drowned') Return Damage;
    
	if (Damage > 25)
	{
		if ( Owner.IsA('PlayerPawn') )
		{ 
			PlayerPawn(Owner).ClientFlash(-0.05,vect(400,400,400));
			PlayerPawn(Owner).PlaySound(DeActivateSound, SLOT_None, 2.7*PlayerPawn(Owner).SoundDampening);
		}
		Charge = 0;
		Destroyed();
	} 
	return 0;
}


function Destroyed()
{
	if ( Owner != None )
	{
		Owner.SetDefaultDisplayProperties();
		if( Owner.Inventory != None )
			Owner.Inventory.SetOwnerDisplay();
	}
	if ( MyEffect != None )
		MyEffect.Destroy();
	Super.Destroyed();
}

function PickupFunction(Pawn Other)
{
	local Inventory I;

	MyEffect = Spawn(class'UT_ShieldBeltEffect', Other,,Other.Location, Other.Rotation); 
	MyEffect.Mesh = Owner.Mesh;
	MyEffect.DrawScale = Owner.Drawscale;
	MyEffect.AmbientGlow = 255;

	if ( Level.Game.bTeamGame && (Other.PlayerReplicationInfo != None) )
		TeamNum = Other.PlayerReplicationInfo.Team;
	else
		TeamNum = 3;
	SetEffectTexture();

	I = Pawn(Owner).FindInventoryType(class'UT_Invisibility');
	if ( I != None )
		MyEffect.bHidden = true;
	
	// remove other armors
	for ( I=Owner.Inventory; I!=None; I=I.Inventory )
		if ( I.bIsAnArmor && (I != self) )
			I.Destroy();
}

function SetEffectTexture()
{
	if ( TeamNum != 3 )
		MyEffect.ScaleGlow = 0.5;
	else
		MyEffect.ScaleGlow = 1.0;
	MyEffect.ScaleGlow *= (0.25 + 0.75 * Charge/Default.Charge);
	if ( TeamFireTextures[TeamNum] == None )
		TeamFireTextures[TeamNum] =FireTexture(DynamicLoadObject(TeamFireTextureStrings[TeamNum], class'Texture'));
	MyEffect.Texture = TeamFireTextures[TeamNum];
	if ( TeamTextures[TeamNum] == None )
		TeamTextures[TeamNum] = Texture(DynamicLoadObject(TeamTextureStrings[TeamNum], class'Texture'));
	MyEffect.LowDetailTexture = TeamTextures[TeamNum];
}

auto state Pickup
{
	simulated function HitWall( Vector HitNormal, Actor Wall )
	{
		// bounce off of wall and reduce velocity
		Velocity -= ( 2 * ( Velocity dot HitNormal ) * HitNormal );
		Velocity /= 1.5;
		if (vSize(Velocity) < 10 && Location == OldLocation)
			SetPhysics( PHYS_Rotating );


		if( VSize( Velocity ) > 100 )
		{
			PlaySound( HitSound1, , 3.0 );
		}
		else if( ( VSize( Velocity ) > 50 ) && ( VSize( Velocity ) < 100 ) )
		{
			PlaySound( HitSound2, , 3.0 );
		}
		else 
		{
			PlaySound( HitSound3, , 3.0 );
		}
	}

	function TakeDamage( int Damage, Pawn EventInstigator, Vector HitLocation, Vector Momentum, Name DamageType )
        {
		local Belt Dropped;

		Dropped = Spawn(Class);
		Dropped.bSimFall = true;
		Dropped.Velocity = Normal( Momentum ) * 2000;
		Dropped.Velocity.Z = 200;
		Dropped.SetPhysics( PHYS_Falling );
		Dropped.SetTimer (RespawnTime, False);
		Dropped.ShieldNum = ShieldNum;
		Dropped.NumPoints = NumPoints;
		Dropped.NumShields = NumShields;
		Destroy();
        }

 
	function Timer()
	{
		if (!PlayerCanSeeMe() || Physics != PHYS_Rotating) 
		{
			SetPhysics( PHYS_Rotating );
			MoveBelt();
			PlaySound( RespawnSound );
		}
		else
			SetTimer (1.0, false);
	}
}

defaultproperties
{
     TeamFireTextureStrings(0)="UnrealShare.Belt_fx.ShieldBelt.RedShield"
     TeamFireTextureStrings(1)="UnrealShare.Belt_fx.ShieldBelt.BlueShield"
     TeamFireTextureStrings(2)="UnrealShare.Belt_fx.ShieldBelt.Greenshield"
     TeamFireTextureStrings(3)="UnrealShare.Belt_fx.ShieldBelt.N_Shield"
     TeamTextureStrings(0)="UnrealShare.Belt_fx.ShieldBelt.newred"
     TeamTextureStrings(1)="UnrealShare.Belt_fx.ShieldBelt.newblue"
     TeamTextureStrings(2)="UnrealShare.Belt_fx.ShieldBelt.newgreen"
     TeamTextureStrings(3)="UnrealShare.Belt_fx.ShieldBelt.newgold"
     TeamFireTextures(0)=FireTexture'UnrealShare.Belt_fx.ShieldBelt.RedShield'
     TeamFireTextures(1)=FireTexture'UnrealShare.Belt_fx.ShieldBelt.BlueShield'
     TeamFireTextures(2)=FireTexture'UnrealShare.Belt_fx.ShieldBelt.Greenshield'
     TeamFireTextures(3)=FireTexture'UnrealShare.Belt_fx.ShieldBelt.N_Shield'
     TeamTextures(0)=Texture'UnrealShare.Belt_fx.ShieldBelt.newred'
     TeamTextures(1)=Texture'UnrealShare.Belt_fx.ShieldBelt.newblue'
     TeamTextures(2)=Texture'UnrealShare.Belt_fx.ShieldBelt.newgreen'
     TeamTextures(3)=Texture'UnrealShare.Belt_fx.ShieldBelt.newgold'
     bDisplayableInv=True
     PickupMessage="You got the Shield Belt."
     ItemName="ShieldBelt"
     RespawnTime=20.000000
     PickupViewMesh=LodMesh'Botpack.ShieldBeltMeshM'
     ProtectionType1=ProtectNone
     ProtectionType2=ProtectNone
     Charge=150
     ArmorAbsorption=100
     bIsAnArmor=True
     AbsorptionPriority=10
     MaxDesireability=3.000000
     PickupSound=Sound'UnrealShare.Pickups.BeltSnd'
     DeActivateSound=Sound'UnrealShare.Pickups.Sbelthe2'
     HitSound1=Sound'InstaSniper.HitWallSound1'
     HitSound2=Sound'InstaSniper.HitWallSound2'
     HitSound3=Sound'InstaSniper.HitWallSound3'
     Icon=Texture'UnrealShare.Icons.I_ShieldBelt'
     bOwnerNoSee=True
     Physics=PHYS_Rotating
     RemoteRole=ROLE_DumbProxy
     Mesh=LodMesh'Botpack.ShieldBeltMeshM'
     CollisionRadius=25.000000
     CollisionHeight=50.000000
     RotationRate=(Yaw=10000)
     bProjTarget=True
     bBounce=True
}
