//=============================================================================
// InstaSniperMut.
//=============================================================================
class InstaSniperMut extends Mutator config(InstaSniper);

var travel ammo AmmoType;
var config int ShieldAmount;
var config int StartingAmmoAmount;
var config int PickupAmmoAmount;
var config int MaxAmmoAmount;
var config bool bHardCoreFrags;
var bool NoConfig;
var bool bPlayer;

function PostBeginPlay()
{
	local int i;
	local Belt MyBelt;
	local AssaultRandomizer AR;

	Super.PostBeginPlay();
	Level.Game.bCoopWeaponMode = false;
	if (ShieldAmount > 15)
		ShieldAmount = 15;
	if (!NoConfig)
		SaveConfig();
	if (DeathMatchPlus(Level.Game).BotConfigType.default.Difficulty >= 6) 
		Spawn(class'ShootTarget');

	ForEach AllActors(class'AssaultRandomizer', AR)
	{
		AR.Tag = 'YYYYYYYY';
	}

	for(i=0;i<ShieldAmount;i++)
	{
		MyBelt = Spawn(class'Belt', , , vect(0,0,-33000));
		MyBelt.ShieldNum = i;
		MyBelt.NumShields = ShieldAmount;
		MyBelt.MoveBelt();
	}

	Level.Game.RegisterDamageMutator( Self );
}

function MutatorTakeDamage( out int ActualDamage, Pawn Victim, Pawn InstigatedBy, out Vector HitLocation, 
						out Vector Momentum, name DamageType)
{
	if ( DamageType == 'Impact' && Victim != InstigatedBy ) 
 	{ 
		ActualDamage = 200; 
		return; 
	} 

	if( Victim.IsA('Bot') && (Victim.health - ActualDamage > 40 ) )
	{ 
		ActualDamage = 0;
		return; 
	}

	if( ( Victim.health - ActualDamage > 0) && (Victim.health - ActualDamage < 100 ) ) 
	{
		ActualDamage = 1000; 
		return; 
	}
}

function bool CheckReplacement(Actor Other, out byte bSuperRelevant)
{
        if ( Other.IsA('TournamentHealth') || Other.IsA('UT_Shieldbelt')
        || Other.IsA('Armor2') || Other.IsA('ThighPads') || Other.IsA('UT_Jumpboots') )
        return false;

        if ( Other.IsA('UDamage') ||  Other.IsA('UT_Invisibility') )
        {
                ReplaceWith(Other,"InstaSniper.TSniperRifle");
                return false;
        }

        if ( Other.IsA('Weapon') && !Other.IsA('TSniperRifle') && !Other.IsA('Translocator') && !Other.IsA('ImpactHammer') )
        {
                ReplaceWith(Other,"InstaSniper.TSniperRifle");
                return false;
        }

        if ( Other.IsA('Ammo') && !Other.IsA('InstAmmo') )
        {
                return false;
        }

        if ( Other.IsA('ImpactHammer') )
        {
                ImpactHammer(Other).bCanThrow = false;
		ImpactHammer(Other).TensionSound = None;
	}

        if ( Other.IsA('TSniperRifle') ) 
        {
                TSniperRifle(Other).PickupAmmoCount = PickupAmmoAmount;
        }

        if ( Other.IsA('Translocator') && !Other.IsA('Tror') && bPlayer ) 
        { 
                return false; 
        }

        if ( Other.IsA('ZoneInfo') )
                ZoneInfo(Other).DamagePerSec = 60; 

	bSuperRelevant = 0;
	return true;
}

function Class<Weapon> MutatedDefaultWeapon()
{
	return None;
}

function ModifyPlayer(Pawn Other) // called by GameInfo.RestartPlayer()
{ 
        AmmoType = Spawn(class'InstAmmo'); 
        Other.AddInventory(AmmoType);   
        AmmoType.BecomeItem(); 
        AmmoType.AmmoAmount = StartingAmmoAmount;   
        AmmoType.MaxAmmo = MaxAmmoAmount;     
        AmmoType.GotoState('Idle2');

	if (Other.IsA('Bot'))
	{
		DeathMatchPlus(Level.Game).GiveWeapon(Other,"Botpack.ImpactHammer");
		if (DeathMatchPlus(Level.Game).bUseTranslocator) 
			bPlayer = false;
		else
			bPlayer = true;
		DeathMatchPlus(Level.Game).GiveWeapon(Other,"InstaSniper.TSniperRifle");
	}
	if (Other.IsA('PlayerPawn'))
	{
		bPlayer = true;
		if (DeathMatchPlus(Level.Game).bUseTranslocator)
			DeathMatchPlus(Level.Game).GiveWeapon(Other,"InstaSniper.Tror");
		if (DeathMatchPlus(Level.Game).BotConfigType.default.Difficulty >= 6) 
			Other.Visibility = 255;
		DeathMatchPlus(Level.Game).GiveWeapon(Other,"InstaSniper.TSniperRifle");
		DeathMatchPlus(Level.Game).GiveWeapon(Other,"InstaSniper.Hammer");
		Other.SwitchToBestWeapon();
	}
        
	if ( NextMutator != None )
		NextMutator.ModifyPlayer(Other); 
}

function bool HandlePickupQuery(Pawn Other, Inventory item, out byte bAllowPickup)
{
        if (Other.FindInventoryType( class'InstAmmo' ) == None)
                return true;
	if ( NextMutator != None )
		return NextMutator.HandlePickupQuery(Other, item, bAllowPickup);
	return false;
}

function ScoreKill(pawn Killer, pawn Other)
{
        if (bHardcoreFrags)
        {  
                if ((Killer != Other) && (Other != None) && (Killer != None))
                {  
                        if ( Killer.isA('bot') )
                        {  
                                Killer.PlayerReplicationInfo.Score += 1;
                                if (Level.Game.Class == Class'TeamGamePlus')
                                {
                                        TeamGamePlus(Level.Game).Teams[Killer.PlayerReplicationInfo.Team].Score += 1;
                                }
                        }
                }
        }
      
        Super.ScoreKill(Killer, Other);  
}

defaultproperties
{
     ShieldAmount=3
     StartingAmmoAmount=2
     PickupAmmoAmount=2
     MaxAmmoAmount=12
     bHardCoreFrags=false
}